/**
 * @author Karl
 * @
 */
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Random;
import java.util.Vector;

import org.seamcat.model.plugin.*;

import seamcat.plugin.propagationmodel.HataSE21Model;

public class ShowLBTapproachR3a implements PostProcessingPlugin, ConsistencyCheck {

	private HataSE21Model						hata									= new HataSE21Model();
	private PluginDistribution					defaultDistribution;
	private boolean								checkPropagationModel					= true;
	private Boolean								simualteLBT								= false;
	private Double								dutyCycle								= 1.;
	private Double								packetDuration							= 250.;
	private Double								listenTimeLBT							= 0.1;
	private Double								thresholdILT							= -90.;
	private Integer								numberOfLinksToConsider					= 1;
	private int									numberOfSystemsSimulated;
	private double								intervalTime;
	private String								filename								= "simulateLBT";
	private String								defaultPath								= "C:";
	private Boolean								shouldBeSaved							= false;

	private ArrayList<LBTcomponent>				simulatedTX								= new ArrayList<LBTcomponent>();
	private ArrayList<LBTcomponent>				simulatedTXcollision					= new ArrayList<LBTcomponent>();
	private ArrayList<LBTcomponent>				simulatedTXdropped						= new ArrayList<LBTcomponent>();
	private LBTcomponent						rSimulatedTX							= new LBTcomponent();
	private ArrayList<Data_to_export_to_XML>	exportData								= new ArrayList<Data_to_export_to_XML>();
	private Data_to_export_to_XML				rExportData								= new Data_to_export_to_XML();
	private BufferedWriter						out;
	private Boolean								simulateLBTvlt							= false;
	private Double								thresholdVLT							= -70.;
	private int									numberOfDroppedTotal					= 0;
	private int									numberOfDroppedCollision				= 0;
	private int									numberOfDroppedILT						= 0;
	private int									numberOfDroppedVLT						= 0;
	private Boolean								simulateVariablePacket					= false;
	private Double								packetLengthVariation					= 5.;
	private double								earliestStart							= Double.MAX_VALUE;						// 
	private double								latestStop								= Double.MIN_VALUE;
	private Random								stddev									= new Random();
	private Integer								indexSelectCalculationModeTimeInerval	= 0;										//duty cycle/packet length
	private int									indexIntervalTimeSelected				= 0;
	private Vector<String>						caclulationModesIntervalTime;

	public void init(ParameterFactory param) {
		defaultDistribution = param.createDistribution(PluginDistribution.DistributionType.DiscreteUniform, 0, 360);
		caclulationModesIntervalTime = new Vector<String>();
		caclulationModesIntervalTime.add("packet length/duty cycle"); // 0
		caclulationModesIntervalTime.add("random packet length/duty cycle"); // 1
		caclulationModesIntervalTime.add("packet length extended by max of packetLengthVariation/duty cycle"); // 2
		caclulationModesIntervalTime.add("latestStop - earliestStart"); //3
	}

	public void cleanUp() {
		// TODO Auto-generated method stub
		exportData.clear();
		simulatedTX.clear();
		simulatedTXcollision.clear();
		simulatedTXdropped.clear();
	}

	public String getDescription() {
		return "This plug-in is to test a first approach of simulating LBT together with duty cycle";
	}

	public void process(ScenarioInfo scenario) {
		// initialize lists for each snapshot
		simulatedTX = new ArrayList<LBTcomponent>();
		simulatedTXcollision = new ArrayList<LBTcomponent>();
		simulatedTXdropped = new ArrayList<LBTcomponent>();

		if (scenario.getCurrentEventNumber() < 2) initPlugin();

		collectDataOfSimulatedTX(scenario);
		findCollision(scenario);
		//		removeDoubleEntries(simulatedTXcollision);
		if (simulatedTXcollision.size() > 1) {
			if (simualteLBT) performLBTmechanism(scenario);
			if (simulateLBTvlt) performSensingVLT(scenario);

			updateSimulatedTX();
		}

		calculateResultingIRSS(scenario);
		if (shouldBeSaved) {
			numberOfDroppedTotal += getNumberOfNotActive();
			numberOfDroppedCollision += getNumberOfDroppedCollision();
			numberOfDroppedILT += getNumberDroppedLBTilt();
			numberOfDroppedVLT += getNumberDroppedLBTvlt();
			collectDataForExport(scenario);
			if (scenario.getCurrentEventNumber() == scenario.getTotalNumberOfEvents()) {// last snapshot
				int numberOfEventsConsidered = scenario.getTotalNumberOfEvents();
				exportResult(numberOfEventsConsidered);
				// exportXSDschema(); there is still an error on the schema
			}
		}
		if (scenario.getCurrentEventNumber() == scenario.getTotalNumberOfEvents()) {
			System.out.println(latestStop - earliestStart);
			System.out.println(packetDuration + packetLengthVariation * 2.57);
		}
	}

	/**
	 * initializes the parameters
	 */
	private void initPlugin() {
		indexIntervalTimeSelected = indexSelectCalculationModeTimeInerval; // takes the index of the calculation mode
		numberOfDroppedTotal = 0;
		numberOfDroppedCollision = 0;
		numberOfDroppedILT = 0;
		numberOfDroppedVLT = 0;
		earliestStart = Double.MAX_VALUE;
		latestStop = Double.MIN_VALUE;
		simulatedTX = new ArrayList<LBTcomponent>();
		simulatedTXcollision = new ArrayList<LBTcomponent>();
		simulatedTXdropped = new ArrayList<LBTcomponent>();
	}

	/**
	 * simulates the sensing of the ILT; at that stage only Extended Hata considered as propagation model between ILT
	 * and VLT
	 * 
	 * @param scenario
	 */
	private void performSensingVLT(ScenarioInfo scenario) {
		Transceiver vlt = scenario.getVictimLink().getTransmitter();
		double rDRSS = scenario.getVictimLink().getDRSSValue();

		double vltPower; // not directly possible as this is not available thought the interface
		vltPower = rDRSS + scenario.getVictimLink().getPathloss();
		double rHTx = vlt.getAntennaHeight();
		hata.setVariationsSelected(false); // disables variations
		int stop = simulatedTXcollision.size();
		for (int i = 0; i < stop; i++) {
			double rDist = simulatedTXcollision.get(i).ilt.getDistanceTo(vlt);
			double rFreq = scenario.getInterferingLink(simulatedTXcollision.get(i).originLink).getFrequency();
			double rHRx = simulatedTXcollision.get(i).ilt.getAntennaHeight();
			double loss = hata.evaluate(rFreq, rDist, rHTx, rHRx);

			double receivedPower = vltPower - loss;
			if (receivedPower < thresholdVLT) {
				simulatedTXcollision.get(i).status = 1; // is transmitting
			}
			else {
				simulatedTXcollision.get(i).status = 0; // is not transmitting
				simulatedTXcollision.get(i).droppedAfterLBTvlt = 1;
				simulatedTXdropped.add(simulatedTXcollision.get(i)); // collects the dropped transmitters
				simulatedTXcollision.remove(i); // removes the dropped from collision
				stop--; // considers the reduced size
			}
		}
	}

	/**
	 * updates the status after LBT, i.e. resets to zero if dropped due to LBT;
	 */

	private void updateSimulatedTX() { // a shorter form as the id corresponds to the index of simulatedTX
		for (int i = 0; i < simulatedTXdropped.size(); i++) {
			rSimulatedTX = simulatedTXdropped.get(i);
			simulatedTX.set(rSimulatedTX.id - 1, rSimulatedTX); // copies the complete dropped information
		}
	}

	/**
	 * calculates the resulting iRSS by considering an additional path loss of 200 dB for inactive transmitters, and
	 * changes the iRSSune and iRSSblock of the origin link accordingly
	 * 
	 * @param scenario
	 */
	private void calculateResultingIRSS(ScenarioInfo scenario) {
		double[] sumUnw = new double[numberOfSystemsSimulated + 1]; // considers one more to ease the counting of the loops
		double[] sumBlock = new double[numberOfSystemsSimulated + 1];
		for (int i = 0; i < simulatedTX.size(); i++) {
			for (int j = 1; j <= numberOfSystemsSimulated; j++) {
				if (simulatedTX.get(i).originLink == j) {
					if (simulatedTX.get(i).status == 0) {
						simulatedTX.get(i).iRSSunw -= 200;
						simulatedTX.get(i).iRSSblock -= 200;
					}
					sumUnw[j] += Math.pow(10, simulatedTX.get(i).iRSSunw / 10);
					sumBlock[j] += Math.pow(10, simulatedTX.get(i).iRSSblock / 10);
				}
			}
		}

		double iRSSchangedUnw = 0;
		double iRSSchangedBlock = 0;
		for (int k = 1; k <= numberOfSystemsSimulated; k++) {
			iRSSchangedUnw = 10 * Math.log10(sumUnw[k]);
			scenario.getProcessingResults().setIRSSUnwanted(k, iRSSchangedUnw);
			iRSSchangedBlock = 10 * Math.log10(sumBlock[k]);
			scenario.getProcessingResults().setIRSSBlocking(k, iRSSchangedBlock);
		}
	}

	/**
	 * collects all data of the simulated interfering transmitters and leaves the default status 0 = inactive; at the
	 * same time a random start time within the interval time is created and depending on the packet duration also the
	 * stop time of each transmitter.
	 * 
	 * @param scenario
	 */
	private void collectDataOfSimulatedTX(ScenarioInfo scenario) {
		for (int i = 1; i <= scenario.getNumberOfInterferingLinks(); i++) {
			simulatedTXdropped = new ArrayList<LBTcomponent>();
			if (scenario.getInterferingLink(i).getOrigLinkIndex() <= numberOfLinksToConsider) {

				rSimulatedTX = new LBTcomponent(); // reset
				rSimulatedTX.id = i;
				numberOfSystemsSimulated = scenario.getInterferingLink(i).getOrigLinkIndex(); // considers automatically the last simulated interfering system
				rSimulatedTX.originLink = numberOfSystemsSimulated;
				rSimulatedTX.ilt = scenario.getInterferingLink(i).getTransmitter();
				rSimulatedTX.ilr = scenario.getInterferingLink(i).getReceiver();
				rSimulatedTX.dutyCycle = dutyCycle;
				rSimulatedTX.lbtThreshold = thresholdILT;
				rSimulatedTX.status = 0;
				if (getSimulateVariablePacket()) {
					double rPacket = packetDuration;
					double rVariation = (rPacket * packetLengthVariation / 100) * stddev.nextGaussian();
					rPacket += rVariation;
					rSimulatedTX.intervalTime = rPacket / dutyCycle * 100;
				}
				else {
					rSimulatedTX.intervalTime = packetDuration / dutyCycle * 100;
				}
				rSimulatedTX.txOnTime = Math.random() * (rSimulatedTX.intervalTime - packetDuration);
				rSimulatedTX.txOffTime = rSimulatedTX.txOnTime + packetDuration;
				earliestStart = Math.min(earliestStart, rSimulatedTX.txOnTime); // to be used for setting the slot for collision
				latestStop = Math.max(latestStop, rSimulatedTX.txOffTime);
				rSimulatedTX.iRSSunw = scenario.getInterferingLink(i).getIRSSUnwanted();
				rSimulatedTX.iRSSblock = scenario.getInterferingLink(i).getIRSSBlocking();
				simulatedTX.add(rSimulatedTX);
			}
		}
	}

	/**
	 * checks all transmitters whether they want to transmit simultaneously and stores them onto simulatedTXcollision by
	 * setting the status to 1 = active
	 * 
	 * @param scenario
	 */
	private void findCollision(ScenarioInfo scenario) {

		switch (getIndexSelectCalculationModeTimeInerval()) {
			case 0: {
				intervalTime = packetDuration / dutyCycle * 100;
				break;
			}
			case 1: {
				double rPacket = packetDuration;				
				double rVariation = rPacket*packetLengthVariation* stddev.nextGaussian()/100;
				rPacket += rVariation;
				intervalTime = rPacket / dutyCycle * 100;
				break;
			}
			case 2: {
				intervalTime = (packetDuration + packetLengthVariation * 2.57) / dutyCycle * 100; 
				// 2.57 corresponds to a probability of about 0.5% if the inverse cumulative distribution function				
				break;
			}
			case 3: {
				intervalTime = (latestStop - earliestStart);
				break;
			}
			default: {
				intervalTime = packetDuration / dutyCycle * 100;
			}

		}

		double checkTime = Math.random() * intervalTime; // a random slot within the time frame of intervalTime
		if (indexIntervalTimeSelected == 3) {// a random time between earliestStart and latestStop
			while (checkTime <= earliestStart || checkTime >= latestStop) {
				checkTime = Math.random() * intervalTime;
			}
		}

		for (int i = 0; i < simulatedTX.size(); i++) {
			if (simulatedTX.get(i).txOnTime < checkTime && simulatedTX.get(i).txOffTime > checkTime) {
				simulatedTX.get(i).status = 1;
				simulatedTXcollision.add(simulatedTX.get(i));
			}
			else {
				simulatedTX.get(i).droppedCollision = 1; // flag which could be used for statistics
			}
		}
	}

	/**
	 * checks whether the received power exceeds the threshold by calculating the path losses of each combination, i.e.
	 * TX1 with TX2 ... TXn, TX2 with TX3 ... TXn and so on; in case the received power exceeds the limit, this
	 * transmitter is considered inactive, otherwise active. At this first trial only Extended Hata outdoor is used for
	 * the path between the transmitters
	 * 
	 * @param scenario
	 */
	private void performLBTmechanism(ScenarioInfo scenario) {
		hata.setVariationsSelected(false);
		double distance = 0;
		int stop = simulatedTXcollision.size();
		for (int i = 0; i < simulatedTXcollision.size() - 1; i++) {
			for (int j = i + 1; j < simulatedTXcollision.size(); j++) {
				distance = simulatedTXcollision.get(i).ilt.getDistanceTo(simulatedTXcollision.get(j).ilt);
				double freq = scenario.getInterferingLink(simulatedTXcollision.get(i).originLink).getFrequency();
				double rHTx = simulatedTXcollision.get(i).ilt.getAntennaHeight();
				double rHRx = simulatedTXcollision.get(j).ilt.getAntennaHeight();
				double loss = hata.evaluate(freq, distance, rHTx, rHRx);
				double receivedPower = simulatedTXcollision.get(i).ilt.getSuppliedPower() - loss;
				if (receivedPower < simulatedTXcollision.get(i).lbtThreshold) {
					simulatedTXcollision.get(i).status = 1; // is transmitting
				}
				else {
					simulatedTXcollision.get(i).status = 0; // is not transmitting
					simulatedTXcollision.get(i).droppedAfterLBTilt = 1;
					simulatedTXdropped.add(simulatedTXcollision.get(i)); // collects the dropped transmitters
					simulatedTXcollision.remove(i); // removes the dropped from collision
					stop--; // considers the reduced size
				}
			}
		}
	}

	/**
	 * assigns only those data to the list to be exported as XML
	 * 
	 * @param scenario
	 */
	private void collectDataForExport(ScenarioInfo scenario) {
		rExportData = new Data_to_export_to_XML();
		rExportData.event = scenario.getCurrentEventNumber();
		rExportData.dropped = getNumberOfNotActive();
		rExportData.droppedDueToLBTilt = getNumberDroppedLBTilt();
		rExportData.droppedDueToLBTvlt = getNumberDroppedLBTvlt();
		exportData.add(rExportData);
	}

	/**
	 * 
	 * @return number of ILT dropped after sensing ILT
	 */
	private int getNumberDroppedLBTvlt() {
		int numberOfdropped = 0;
		for (int i = 0; i < simulatedTX.size(); i++) {
			if (simulatedTX.get(i).droppedAfterLBTvlt != 0) numberOfdropped++;
		}
		return numberOfdropped;
	}

	/**
	 * 
	 * @return number of ILT dropped after performing LBT with all active ILT
	 */
	private int getNumberDroppedLBTilt() {
		int numberOfdropped = 0;
		for (int i = 0; i < simulatedTX.size(); i++) {
			if (simulatedTX.get(i).droppedAfterLBTilt != 0) numberOfdropped++;
		}
		return numberOfdropped;
	}

	/**
	 * counts the number of inactive transmitters
	 * 
	 * @return dropped
	 */
	private int getNumberOfNotActive() {
		int dropped = 0;
		for (int i = 0; i < simulatedTX.size(); i++) {
			if (simulatedTX.get(i).status == 0) dropped++;
		}
		return dropped;
	}

	/**
	 * saves the collected data onto an XML file
	 * 
	 * @param numberOfSimulatedEvents
	 */
	private void exportResult(int numberOfSimulatedEvents) {
		String pathToSave = defaultPath + "\\" + filename + ".xml";
		try {
			out = new BufferedWriter(new FileWriter(pathToSave));
			String xmlDeclaration = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>";
			String xmlRoot = "result";
			String xmlElement = "snapshot";
			out.write(xmlDeclaration);
			out.newLine();

			out.write("<" + xmlRoot + ">");
			out.newLine();
			String element = "";
			element = "<calculationMethod";
			element += " intervalTime = \"" + caclulationModesIntervalTime.get(getIndexSelectCalculationModeTimeInerval());
			element += "\">";
			element += "</calculationMethod>";
			out.write(element);
			out.newLine();
			element = "<general ";
			element += " dutyCycle = \"" + dutyCycle + " %";
			element += " \" packetLength = \"" + packetDuration;
			if (simulateVariablePacket) element += " \" packetVariation = \"" + packetLengthVariation + " %";
			element += "\"  thresholdILT = \"" + thresholdILT + " dBm";
			if (simulateLBTvlt) element += "\"  thresholdVLT = \"" + thresholdVLT + " dBm";
			element += "\">";
			element += "</general>";
			out.write(element);
			out.newLine();
			element = "<summary";
			element += " simulatedLinks = \"" + numberOfSystemsSimulated;
			element += "\" numberOfTXsimulated = \"" + simulatedTX.size();
			element += "\" totalNumberOfTXsimulated = \"" + simulatedTX.size() * numberOfSimulatedEvents;	
			element += "\" totalNumberOfTXdroppedLBTcollision = \"" + getNumberOfDroppedCollision();
			element += "\" totalNumberOfTXdroppedLBTilt = \"" + numberOfDroppedILT;
			element += "\" totalNumberOfTXdroppedLBTvlt = \"" + numberOfDroppedVLT;
			element += "\">";
			element += "</summary>";
			out.write(element);
			out.newLine();

			for (int i = 0; i < exportData.size(); i++) {
				element = "<" + xmlElement;
				element += " eventNr = \"" + exportData.get(i).event;
				element += " \" droppedTx = \"" + exportData.get(i).dropped;
				element += " \" droppedTxLBTilt = \"" + exportData.get(i).droppedDueToLBTilt;
				element += " \" droppedTxLBTvlt = \"" + exportData.get(i).droppedDueToLBTvlt + "\">";
				element += "</" + xmlElement + ">";
				out.write(element);
				out.newLine();
			}
			out.write("</" + xmlRoot + ">");
			out.flush();
			out.close();

		} catch (IOException e) {
			defaultPath = "C:"; // valid path before re-starting this method
			exportResult(numberOfSimulatedEvents);
		}
	}

	/**
	 * generates the DTD-Schema to be applied to the XML file
	 * 
	 * @throws IOException
	 */
	private void exportXSDschema() {
		// TODO Auto-generated method stub
		String pathToSave = defaultPath + "\\simulateLBT.xsd";
		String schema = "";

		try {
			BufferedWriter outXSD = new BufferedWriter(new FileWriter(pathToSave));
			schema += "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
			schema += "<xs:schema xmlns:xs=\"http://www.w3.org/2001/XMLSchema\" elementFormDefault=\"qualified\" attributeFormDefault=\"unqualified\"><xs:element name=\"result\" type=\"resultType\"><xs:annotation><xs:documentation>Comment describing your root element</xs:documentation>";
			schema += "</xs:annotation></xs:element>";
			schema += "xs:element name=\"general\"><xs:complexType><xs:attribute name=\"dutyCycle\"/><xs:attribute name=\"packetLength\"/><xs:attribute name=\"packetVariation\"/><xs:attribute name=\"threshold\"/></xs:complexType></xs:element>";
			schema += "<xs:element name=\"snapshot\"><xs:complexType><xs:attribute name=\"eventNr\"/><xs:attribute name=\"droppedTx\"/><xs:attribute name=\"droppedTxLBTilt\"/><xs:attribute name=\"droppedTxLBTvlt\"/></xs:complexType></xs:element>";
			schema += "xs:element name=\"summary\"><xs:complexType><xs:attribute name=\"totalNumberOfTX\"/><xs:attribute name=\"totalNumberOfTXdropped\"/><xs:attribute name=\"totalNumberOfTXdroppedLBTvlt\"/></xs:complexType></xs:element>";
			schema += "<xs:complexType name=\"resultType\"><xs:sequence><xs:element ref=\"general\"/><xs:element ref=\"summary\" minOccurs=\"0\"/><xs:element ref=\"snapshot\" maxOccurs=\"unbounded\"/></xs:sequence></xs:complexType>";
			schema += "</xs:schema>";
			outXSD.write(schema);
			outXSD.flush();
			outXSD.close();
		} catch (Exception e) {
			// TODO: handle exception
		}

	}

	public int getNumberOfParameters() {
		return 14;
	}

	public ParameterType getParameterType(int id) {
		switch (id) {
			case 1: {
				return ParameterType.Boolean;
			}
			case 2: {
				return ParameterType.Double;
			}

			case 3: {
				return ParameterType.Double;

			}
			case 4: {
				return ParameterType.Double;
			}
			case 5: {
				return ParameterType.Double;
			}
			case 6: {
				return ParameterType.Integer;
			}
			case 7: {
				return ParameterType.String; // filename for the output
			}
			case 8: {
				return ParameterType.String; // default path for the output files
			}
			case 9: {
				return ParameterType.Boolean; // flag to save output
			}
			case 10: {
				return ParameterType.Boolean; // flag for LBT of VLT
			}
			case 11: {
				return ParameterType.Double;
			}
			case 12: {
				return ParameterType.Boolean; // flag for variable packet length
			}
			case 13: {
				return ParameterType.Double;
			}
			case 14: {
				return ParameterType.Integer;
			}

			default: {
				throw new IllegalArgumentException("illegal parameter id");
			}

		}
	}

	public void setParameterValue(int id, Object value) {
		try {
			switch (id) {
				case 1: {
					simualteLBT = (Boolean) value;
					break;
				}
				case 2: {
					dutyCycle = (Double) value;
					break;
				}
				case 3: {
					packetDuration = (Double) value;
					break;
				}
				case 4: {
					listenTimeLBT = (Double) value;
					break;
				}
				case 5: {
					thresholdILT = (Double) value;
					break;
				}
				case 6: {
					numberOfLinksToConsider = (Integer) value;
					break;
				}
				case 7: {
					filename = (String) value;
					break;
				}
				case 8: {
					defaultPath = (String) value;
					break;
				}
				case 9: {
					shouldBeSaved = (Boolean) value;
					break;
				}
				case 10: {
					simulateLBTvlt = (Boolean) value;
					break;
				}
				case 11: {
					thresholdVLT = (Double) value;
					break;
				}
				case 12: {
					simulateVariablePacket = (Boolean) value;
					setSimulateVariablePacket(simulateVariablePacket);
					break;
				}
				case 13: {
					packetLengthVariation = (Double) value;
					break;
				}
				case 14: {
					indexSelectCalculationModeTimeInerval = (Integer) value;
					setIndexSelectCalculationModeTimeInerval(indexSelectCalculationModeTimeInerval);
					break;
				}
				default: {
					throw new IllegalArgumentException("illegal parameter id");
				}
			}
		} catch (Exception e) {

		}
	}

	public Object getParameterValue(int id) {
		switch (id) {
			case 1: {
				return simualteLBT;
			}
			case 2: {
				return dutyCycle;
			}
			case 3: {
				return packetDuration;
			}
			case 4: {
				return listenTimeLBT;
			}
			case 5: {
				return thresholdILT;
			}
			case 6: {
				return numberOfLinksToConsider;
			}
			case 7: {
				return filename;
			}
			case 8: {
				return defaultPath;
			}
			case 9: {
				return shouldBeSaved;
			}
			case 10: {
				return simulateLBTvlt;
			}
			case 11: {
				return thresholdVLT;
			}
			case 12: {
				return simulateVariablePacket;
			}
			case 13: {
				return packetLengthVariation;
			}
			case 14: {
				return indexSelectCalculationModeTimeInerval;
			}
			default: {
				throw new IllegalArgumentException("illegal parameter id");
			}

		}
	}

	public String getParameterName(int id) {
		switch (id) {
			case 1: {
				return "simulate LBT of ILTs";
			}
			case 2: {
				return "duty cycle [% of time interval]";
			}
			case 3: {
				return "packet length [ms]";
			}
			case 4: {
				return "LBT listen time [% of packet length]";
			}
			case 5: {
				return "LBT threshold [dBm]";
			}
			case 6: {
				return "number of interfering systems (IL)";
			}
			case 7: {
				return "file name of the output data [without extension]";
			}

			case 8: {
				return "default path ";
			}
			case 9: {
				return "save output file ";
			}
			case 10: {
				return "simulate sensing of VLT ";
			}
			case 11: {
				return "threshold for sensing of VLT [dBm]";
			}
			case 12: {
				return "simulate variable packet length ";
			}
			case 13: {
				return "random variation of packet length [%]";
			}
			case 14: {
				return "select calculation mode for interval time";
			}
			default: {
				throw new IllegalArgumentException("illegal parameter id");
			}

		}
	}

	/*
	 * check consistency of the workspace
	 */
	public boolean check(ScenarioInfo info) {
		checkPropagationModel = true;

		String warning = "<html>";
		warning += "</html>";

		if (checkPropagationModel == false) {
			info.addConsistencyWarning(warning);
			return false;
		}
		else {
			return true;
		}
	}

	@Override
	public Vector<String> getAllowedParameterValues(int index) {
		// TODO Auto-generated method stub
		switch (index) {
			case 14:
				return caclulationModesIntervalTime;

			default:
				return null;
		}
	}

	public Boolean getSimulateVariablePacket() {
		return simulateVariablePacket;
	}

	public void setSimulateVariablePacket(Boolean simulateVariablePacket) {
		this.simulateVariablePacket = simulateVariablePacket;
	}

	public int getNumberOfDroppedCollision() {
		int rNumberOfDroppedCollision = 0;
		for (int i = 0; i < simulatedTX.size(); i++) {
			if (simulatedTX.get(i).droppedCollision == 1) rNumberOfDroppedCollision++;
		}
		return rNumberOfDroppedCollision;
	}

	public Integer getIndexSelectCalculationModeTimeInerval() {
		return indexSelectCalculationModeTimeInerval;
	}

	public void setIndexSelectCalculationModeTimeInerval(Integer indexSelectCalculationModeTimeInerval) {
		this.indexSelectCalculationModeTimeInerval = indexSelectCalculationModeTimeInerval;
	}

}

class LBTcomponent {
	int						id;
	int						status				= 0;	// 0 = inactive;  changed by analyzing the randomized scenario
	int						linkIndex;					// index within the interfering system
	int						originLink;				// to separate different types of interfering links   
	double					dutyCycle;					// could be assigned depending on different types of interferers? --- global input parameter ??
	double					txOnTime;					// time the tx is simulated to transmit
	double					txOffTime;					// depends on DC and intervalTime
	double					intervalTime;				// maybe a global parameter ??   
	double					packetLength;
	double					startListenTime;			// random, as fraction of the packet length ??
	double					listenTime;				// fraction of the intervalTime, maybe a global parameter ?? 
	double					lbtThreshold;				// maybe a global parameter ??   
	InterferingTransmitter	ilt;						// copy of the interfering transmitter
	Transceiver				ilr;						// copy of ILR, position probably needed for path loss calculations for LBT
	double					iRSSblock;
	double					iRSSunw;
	int						droppedCollision	= 0;
	int						droppedAfterLBTilt	= 0;
	int						droppedAfterLBTvlt;
}

class Data_to_export_to_XML {
	int		event;
	int		dropped;
	int		droppedDueToLBTilt;
	int		droppedDueToLBTvlt;
	int		numberOfTransmitter;
	double	dutyCycle;
	double	packetLength;
	double	threshold;
}
